@@ -33,7 +33,7 @@
 #include "geanylatex.h"
 #include "ctype.h"
 
-PLUGIN_VERSION_CHECK(199)
+PLUGIN_VERSION_CHECK(217)
 
 PLUGIN_SET_TRANSLATABLE_INFO(
 	LOCALEDIR,
@@ -303,7 +303,7 @@ plugin_configure(GtkDialog * dialog)
 	config_widgets.wizard_to_generic_toolbar = gtk_check_button_new_with_label(
 		_("Add a wizard icon to Geany's main toolbar"));
 	config_widgets.lower_selection_on_smallcaps = gtk_check_button_new_with_label(
-		_("Lower selection when formating smallcaps (\\textsc{})"));
+		_("Lower selection when formatting smallcaps (\\textsc{})"));
 
 	config_widgets.glatex_autocompletion_active = gtk_combo_box_new_text();
 	gtk_combo_box_insert_text(GTK_COMBO_BOX(config_widgets.glatex_autocompletion_active), 0,
@@ -501,6 +501,18 @@ static void on_document_filetype_set(G_GNUC_UNUSED GObject *obj, GeanyDocument *
 }
 
 
+static gint get_position_relative(ScintillaObject *sci, gint pos, gint n)
+{
+	return (gint) scintilla_send_message(sci, SCI_POSITIONRELATIVE, (uptr_t) pos, n);
+}
+
+
+static gint get_char_relative(ScintillaObject *sci, gint pos, gint n)
+{
+	return sci_get_char_at(sci, get_position_relative(sci, pos, n));
+}
+
+
 static gboolean on_editor_notify(G_GNUC_UNUSED GObject *object, GeanyEditor *editor,
 									SCNotification *nt, G_GNUC_UNUSED gpointer data)
 {
@@ -688,17 +700,29 @@ static gboolean on_editor_notify(G_GNUC_UNUSED GObject *object, GeanyEditor *edi
 				}
 				default:
 				{
-					if (glatex_capitalize_sentence_starts == TRUE)
+					if (glatex_capitalize_sentence_starts == TRUE &&
+						g_ascii_isspace(get_char_relative(sci, pos, -2)))
 					{
-						if (sci_get_char_at(sci, pos -2) == ' ' &&
-							(sci_get_char_at(sci, pos -3) == '.' ||
-							 sci_get_char_at(sci, pos -3) == '!' ||
-							 sci_get_char_at(sci, pos -3) == '?' ))
+						gint prevNonWhite = 0;
+						gint i;
+
+						/* find the previous non-white character */
+						i = get_position_relative(sci, pos, -3);
+						while (g_ascii_isspace((prevNonWhite = sci_get_char_at(sci, i))))
+						{
+							/* no need to bother about multi-byte characters here since
+							 * we only check for ASCII space characters anyway */
+							--i;
+						}
+
+						if (prevNonWhite == '.' ||
+							prevNonWhite == '!' ||
+							prevNonWhite == '?')
 						{
 							gchar *upperLtr = NULL;
 							gchar *selection = NULL;
 
-							sci_set_selection_start(sci, pos - 1);
+							sci_set_selection_start(sci, get_position_relative(sci, pos, -1));
 							sci_set_selection_end(sci, pos);
 
 							selection = sci_get_selection_contents(sci);
@@ -708,8 +732,8 @@ static gboolean on_editor_notify(G_GNUC_UNUSED GObject *object, GeanyEditor *edi
 							g_free(upperLtr);
 							g_free(selection);
 						}
-						break;
 					}
+					break;
 				}
 			} /* Closing switch  */
 			/* later there could be some else ifs for other keywords */
@@ -2229,7 +2253,7 @@ add_menu_to_menubar(void)
 		menu_latex_insert_command = gtk_menu_item_new_with_mnemonic(
 			_("Insert _Command"));
 		ui_widget_set_tooltip_text(menu_latex_ref,
-			_("Inserting costumized command to document"));
+			_("Inserting customized command to document"));
 		gtk_container_add(GTK_CONTAINER(menu_latex_menu),
 			menu_latex_insert_command);
 		g_signal_connect(menu_latex_insert_command, "activate",
